"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const internal_1 = require("../internal");
class Group extends internal_1.State {
    /**
     * @public
     * Group - Holds Items of Collection
     * @param collection - Collection to that the Group belongs
     * @param initialItems - Initial Key of Items in this Group
     * @param config - Config
     */
    constructor(collection, initialItems, config = {}) {
        super(collection.agileInstance(), initialItems || [], config);
        this._output = []; // Output of Group
        this._items = []; // Items of Group
        this.notFoundItemKeys = []; // Contains all keys of Group that can't be found in Collection
        this.collection = () => collection;
        // Add rebuild to sideEffects to rebuild Group on Value Change
        this.addSideEffect(Group.rebuildGroupSideEffectKey, () => this.rebuild());
        // Initial Rebuild
        this.rebuild();
    }
    /**
     * @public
     * Get Item Values of Group
     */
    get output() {
        internal_1.ComputedTracker.tracked(this.observer);
        return this._output;
    }
    /**
     * @public
     * Set Item Values of Group
     */
    set output(value) {
        this._output = value;
    }
    /**
     * @public
     * Get Items of Group
     */
    get items() {
        internal_1.ComputedTracker.tracked(this.observer);
        return this._items.map((item) => item());
    }
    /**
     * @public
     * Set Items of Group
     */
    set items(value) {
        this._items = value.map((item) => () => item);
    }
    //=========================================================================================================
    // Has
    //=========================================================================================================
    /**
     * @public
     * Checks if Group contains ItemKey
     * @param itemKey - ItemKey that gets checked
     */
    has(itemKey) {
        return this.value.findIndex((key) => key === itemKey) !== -1;
    }
    //=========================================================================================================
    // Size
    //=========================================================================================================
    /**
     * @public
     * Get size of Group (-> How many Items it contains)
     */
    get size() {
        return this.value.length;
    }
    //=========================================================================================================
    // Remove
    //=========================================================================================================
    /**
     * @public
     * Removes ItemKey/s from Group
     * @param itemKeys - ItemKey/s that get removed from Group
     * @param config - Config
     */
    remove(itemKeys, config = {}) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const notExistingItemKeysInCollection = [];
        const notExistingItemKeys = [];
        let newGroupValue = internal_1.copy(this.nextStateValue);
        config = internal_1.defineConfig(config, {
            background: false,
        });
        // Remove ItemKeys from Group
        _itemKeys.forEach((itemKey) => {
            // Check if itemKey exists in Group
            if (!newGroupValue.includes(itemKey)) {
                internal_1.Agile.logger.error(`Couldn't find ItemKey '${itemKey}' in Group '${this._key}'!`);
                notExistingItemKeys.push(itemKey);
                notExistingItemKeysInCollection.push(itemKey);
                return;
            }
            // Check if ItemKey exists in Collection
            if (!this.collection().getItem(itemKey))
                notExistingItemKeysInCollection.push(itemKey);
            // Remove ItemKey from Group
            newGroupValue = newGroupValue.filter((key) => key !== itemKey);
        });
        // Return if passed ItemKeys doesn't exist
        if (notExistingItemKeys.length >= _itemKeys.length)
            return this;
        // If all removed ItemKeys doesn't exist in Collection -> no rerender necessary since output doesn't change
        if (notExistingItemKeysInCollection.length >= _itemKeys.length)
            config.background = true;
        this.set(newGroupValue, { background: config.background });
        return this;
    }
    //=========================================================================================================
    // Add
    //=========================================================================================================
    /**
     * @public
     * Adds ItemKey/s to Group
     * @param itemKeys - ItemKey/s that get added to the Group
     * @param config - Config
     */
    add(itemKeys, config = {}) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const notExistingItemKeysInCollection = [];
        const existingItemKeys = [];
        let newGroupValue = internal_1.copy(this.nextStateValue);
        config = internal_1.defineConfig(config, {
            method: 'push',
            overwrite: false,
            background: false,
        });
        // Add ItemKeys to Group
        _itemKeys.forEach((itemKey) => {
            const existsInGroup = newGroupValue.includes(itemKey);
            // Check if ItemKey exists in Collection
            if (!this.collection().getItem(itemKey))
                notExistingItemKeysInCollection.push(itemKey);
            // Remove ItemKey from Group if it should get overwritten and already exists
            if (existsInGroup) {
                if (config.overwrite) {
                    newGroupValue = newGroupValue.filter((key) => key !== itemKey);
                }
                else {
                    existingItemKeys.push(itemKey);
                    return;
                }
            }
            // Add new ItemKey to Group
            newGroupValue[config.method || 'push'](itemKey);
        });
        // Return if passed ItemKeys already exist
        if (existingItemKeys.length >= _itemKeys.length)
            return this;
        // If all added ItemKeys doesn't exist in Collection or already exist -> no rerender necessary since output doesn't change
        if (notExistingItemKeysInCollection.concat(existingItemKeys).length >=
            _itemKeys.length)
            config.background = true;
        this.set(newGroupValue, { background: config.background });
        return this;
    }
    //=========================================================================================================
    // Replace
    //=========================================================================================================
    /**
     * @public
     * Replaces oldItemKey with newItemKey
     * @param oldItemKey - Old ItemKey
     * @param newItemKey - New ItemKey
     * @param config - Config
     */
    replace(oldItemKey, newItemKey, config = {}) {
        const newGroupValue = internal_1.copy(this._value);
        newGroupValue.splice(newGroupValue.indexOf(oldItemKey), 1, newItemKey);
        this.set(newGroupValue, config);
        return this;
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (internal_1.isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = this._key;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = internal_1.defineConfig(_config, {
            instantiate: true,
            followCollectionPattern: false,
            storageKeys: [],
        });
        if (_config.followCollectionPersistKeyPattern) {
            key = internal_1.CollectionPersistent.getGroupStorageKey(key || this._key, this.collection()._key);
        }
        super.persist(key, {
            instantiate: _config.instantiate,
            storageKeys: _config.storageKeys,
        });
        return this;
    }
    //=========================================================================================================
    // Rebuild
    //=========================================================================================================
    /**
     * @internal
     * Rebuilds Output and Items of Group
     */
    rebuild() {
        const notFoundItemKeys = []; // Item Keys that couldn't be found in Collection
        const groupItems = [];
        // Create groupItems by finding Item at ItemKey in Collection
        this._value.forEach((itemKey) => {
            const item = this.collection().getItem(itemKey);
            if (item)
                groupItems.push(item);
            else
                notFoundItemKeys.push(itemKey);
        });
        // Create groupOutput out of groupItems
        const groupOutput = groupItems.map((item) => {
            return item.getPublicValue();
        });
        // Logging
        if (notFoundItemKeys.length > 0) {
            internal_1.Agile.logger.warn(`Couldn't find some Items in Collection '${this.collection()._key}' (${this._key})`, notFoundItemKeys);
        }
        this.items = groupItems;
        this._output = groupOutput;
        this.notFoundItemKeys = notFoundItemKeys;
    }
}
exports.Group = Group;
Group.rebuildGroupSideEffectKey = 'rebuildGroup';
