"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const internal_1 = require("../internal");
class Collection {
    /**
     * @public
     * Collection - Class that holds a List of Objects with key and causes rerender on subscribed Components
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance, config = {}) {
        this.size = 0; // Amount of Items stored in Collection
        this.data = {}; // Collection Data
        this.isPersisted = false; // If Collection can be stored in Agile Storage (-> successfully integrated persistent)
        this.groups = {};
        this.selectors = {};
        this.isInstantiated = false;
        this.agileInstance = () => agileInstance;
        // Set temp Config for creating proper Placeholder Items (of Selector)
        this.config = {
            defaultGroupKey: 'default',
            primaryKey: 'id',
        };
        // Assign Properties
        let _config = typeof config === 'function' ? config(this) : config;
        _config = internal_1.defineConfig(_config, {
            primaryKey: 'id',
            groups: {},
            selectors: {},
            defaultGroupKey: 'default',
        });
        this._key = _config.key;
        this.config = {
            defaultGroupKey: _config.defaultGroupKey,
            primaryKey: _config.primaryKey,
        };
        this.initialConfig = _config;
        this.initGroups(_config.groups);
        this.initSelectors(_config.selectors);
        if (_config.initialData)
            this.collect(_config.initialData);
        this.isInstantiated = true;
    }
    /**
     * @public
     * Set Key/Name of Collection
     */
    set key(value) {
        this.setKey(value);
    }
    /**
     * @public
     * Get Key/Name of Collection
     */
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @public
     * Set Key/Name of Collection
     * @param value - New Key/Name of Collection
     */
    setKey(value) {
        var _a, _b;
        const oldKey = this._key;
        // Update State Key
        this._key = value;
        // Update Key in Persistent (only if oldKey equal to persistentKey -> otherwise the PersistentKey got formatted and will be set where other)
        if (value && ((_a = this.persistent) === null || _a === void 0 ? void 0 : _a._key) === oldKey)
            (_b = this.persistent) === null || _b === void 0 ? void 0 : _b.setKey(value);
        return this;
    }
    //=========================================================================================================
    // Group
    //=========================================================================================================
    /**
     * @public
     * Group - Holds Items of this Collection
     * @param initialItems - Initial ItemKeys of Group
     * @param config - Config
     */
    Group(initialItems, config) {
        if (this.isInstantiated) {
            const key = (config === null || config === void 0 ? void 0 : config.key) || internal_1.generateId();
            internal_1.Agile.logger.warn("After the instantiation we recommend using 'MY_COLLECTION.createGroup' instead of 'MY_COLLECTION.Group'");
            if (!(config === null || config === void 0 ? void 0 : config.key))
                internal_1.Agile.logger.warn(`Failed to find key for creation of Group. Group with random key '${key}' got created!`);
            return this.createGroup(key, initialItems);
        }
        return new internal_1.Group(this, initialItems, config);
    }
    //=========================================================================================================
    // Selector
    //=========================================================================================================
    /**
     * @public
     * Selector - Represents an Item of this Collection
     * @param initialKey - Key of Item that the Selector represents
     * @param config - Config
     */
    Selector(initialKey, config) {
        if (this.isInstantiated) {
            const key = (config === null || config === void 0 ? void 0 : config.key) || internal_1.generateId();
            internal_1.Agile.logger.warn("After the instantiation we recommend using 'MY_COLLECTION.createSelector' instead of 'MY_COLLECTION.Selector'");
            if (!(config === null || config === void 0 ? void 0 : config.key))
                internal_1.Agile.logger.warn(`Failed to find key for creation of Selector. Selector with random key '${key}' got created!`);
            return this.createSelector(key, initialKey);
        }
        return new internal_1.Selector(this, initialKey, config);
    }
    //=========================================================================================================
    // Init Groups
    //=========================================================================================================
    /**
     * @internal
     * Instantiates Groups
     */
    initGroups(groups) {
        if (!groups)
            return;
        let groupsObject = {};
        // If groups is Array of GroupNames transform it to Group Object
        if (Array.isArray(groups)) {
            groups.forEach((groupKey) => {
                groupsObject[groupKey] = new internal_1.Group(this, [], {
                    key: groupKey,
                });
            });
        }
        else
            groupsObject = groups;
        // Add default Group
        groupsObject[this.config.defaultGroupKey] = new internal_1.Group(this, [], {
            key: this.config.defaultGroupKey,
        });
        // Set Key/Name of Group to property Name
        for (const key in groupsObject)
            if (!groupsObject[key]._key)
                groupsObject[key].setKey(key);
        this.groups = groupsObject;
    }
    //=========================================================================================================
    // Init Selectors
    //=========================================================================================================
    /**
     * @internal
     * Instantiates Selectors
     */
    initSelectors(selectors) {
        if (!selectors)
            return;
        let selectorsObject = {};
        // If selectors is Array of SelectorNames transform it to Selector Object
        if (Array.isArray(selectors)) {
            selectors.forEach((selectorKey) => {
                selectorsObject[selectorKey] = new internal_1.Selector(this, selectorKey, {
                    key: selectorKey,
                });
            });
        }
        else
            selectorsObject = selectors;
        // Set Key/Name of Selector to property Name
        for (const key in selectorsObject)
            if (!selectorsObject[key]._key)
                selectorsObject[key].setKey(key);
        this.selectors = selectorsObject;
    }
    //=========================================================================================================
    // Collect
    //=========================================================================================================
    /**
     * @public
     * Collect Item/s
     * @param data - Data that gets added to Collection
     * @param groupKeys - Add collected Item/s to certain Groups
     * @param config - Config
     */
    collect(data, groupKeys, config = {}) {
        const _data = internal_1.normalizeArray(data);
        const _groupKeys = internal_1.normalizeArray(groupKeys);
        const defaultGroupKey = this.config.defaultGroupKey;
        const primaryKey = this.config.primaryKey;
        config = internal_1.defineConfig(config, {
            method: 'push',
            background: false,
            patch: false,
            select: false,
        });
        // Add default GroupKey, because Items get always added to default Group
        if (!_groupKeys.includes(defaultGroupKey))
            _groupKeys.push(defaultGroupKey);
        // Create not existing Groups
        _groupKeys.forEach((key) => !this.groups[key] && this.createGroup(key));
        _data.forEach((data, index) => {
            const itemKey = data[primaryKey];
            // Add Item to Collection
            const success = this.setData(data, {
                patch: config.patch,
                background: config.background,
            });
            if (!success)
                return this;
            // Add ItemKey to provided Groups
            _groupKeys.forEach((groupKey) => {
                var _a;
                (_a = this.getGroup(groupKey)) === null || _a === void 0 ? void 0 : _a.add(itemKey, {
                    method: config.method,
                    background: config.background,
                });
            });
            if (config.select)
                this.createSelector(itemKey, itemKey);
            if (config.forEachItem)
                config.forEachItem(data, itemKey, index);
        });
        return this;
    }
    //=========================================================================================================
    // Update
    //=========================================================================================================
    /**
     * @public
     * Updates Item at provided Key
     * @param itemKey - ItemKey of Item that gets updated
     * @param changes - Changes that will be merged into the Item (flatMerge)
     * @param config - Config
     */
    update(itemKey, changes, config = {}) {
        const item = this.getItem(itemKey, { notExisting: true });
        const primaryKey = this.config.primaryKey;
        config = internal_1.defineConfig(config, {
            addNewProperties: false,
            background: false,
        });
        if (!item) {
            internal_1.Agile.logger.error(`Item with key/name '${itemKey}' doesn't exist in Collection '${this._key}'!`);
            return undefined;
        }
        if (!internal_1.isValidObject(changes)) {
            internal_1.Agile.logger.error(`You have to pass an valid Changes Object to update '${itemKey}' in '${this._key}'!`);
            return undefined;
        }
        const oldItemKey = item._value[primaryKey];
        const newItemKey = changes[primaryKey] || oldItemKey;
        const updateItemKey = oldItemKey !== newItemKey;
        // Delete primaryKey from 'changes' because if it has changed, it gets properly updated in 'updateItemKey' (below)
        if (changes[primaryKey])
            delete changes[primaryKey];
        // Update ItemKey
        if (updateItemKey)
            this.updateItemKey(oldItemKey, newItemKey, {
                background: config.background,
            });
        // Apply changes to Item
        item.patch(changes, {
            background: config.background,
            addNewProperties: config.addNewProperties,
        });
        return item;
    }
    //=========================================================================================================
    // Create Group
    //=========================================================================================================
    /**
     * @public
     * Creates new Group that can hold Items of Collection
     * @param groupKey - Name/Key of Group
     * @param initialItems - Initial ItemKeys of Group
     */
    createGroup(groupKey, initialItems = []) {
        let group = this.getGroup(groupKey, { notExisting: true });
        if (!this.isInstantiated) {
            internal_1.Agile.logger.warn("We recommend to use 'MY_COLLECTION.Group' instead of 'MY_COLLECTION.createGroup' in the Collection config!");
        }
        // Check if Group already exists
        if (group) {
            if (!group.isPlaceholder) {
                internal_1.Agile.logger.warn(`Group with the name '${groupKey}' already exists!`);
                return group;
            }
            group.set(initialItems, { overwrite: true });
            return group;
        }
        // Create Group
        group = new internal_1.Group(this, initialItems, { key: groupKey });
        this.groups[groupKey] = group;
        return group;
    }
    //=========================================================================================================
    // Has Group
    //=========================================================================================================
    /**
     * @public
     * Check if Group exists in Collection
     * @param groupKey - Key/Name of Group
     * @param config - Config
     */
    hasGroup(groupKey, config = {}) {
        return !!this.getGroup(groupKey, config);
    }
    //=========================================================================================================
    // Get Group
    //=========================================================================================================
    /**
     * @public
     * Get Group by Key/Name
     * @param groupKey - Key/Name of Group
     * @param config - Config
     */
    getGroup(groupKey, config = {}) {
        config = internal_1.defineConfig(config, {
            notExisting: false,
        });
        // Get Group
        const group = groupKey ? this.groups[groupKey] : undefined;
        // Check if Group exists
        if (!group || (!config.notExisting && group.isPlaceholder))
            return undefined;
        internal_1.ComputedTracker.tracked(group.observer);
        return group;
    }
    //=========================================================================================================
    // Get Group With Reference
    //=========================================================================================================
    /**
     * @public
     * Get Group by Key/Name or a Reference to it if it doesn't exist yet
     * @param groupKey - Name/Key of Group
     */
    getGroupWithReference(groupKey) {
        let group = this.getGroup(groupKey, { notExisting: true });
        // Create dummy Group to hold reference
        if (!group) {
            group = new internal_1.Group(this, [], {
                key: groupKey,
                isPlaceholder: true,
            });
            this.groups[groupKey] = group;
        }
        internal_1.ComputedTracker.tracked(group.observer);
        return group;
    }
    //=========================================================================================================
    // Remove Group
    //=========================================================================================================
    /**
     * @public
     * Removes Group by Key/Name
     * @param groupKey - Name/Key of Group
     */
    removeGroup(groupKey) {
        if (!this.groups[groupKey]) {
            internal_1.Agile.logger.warn(`Group with the key/name '${groupKey}' doesn't exist!`);
            return this;
        }
        delete this.groups[groupKey];
        return this;
    }
    //=========================================================================================================
    // Create Selector
    //=========================================================================================================
    /**
     * @public
     * Creates new Selector that represents an Item of the Collection
     * @param selectorKey - Name/Key of Selector
     * @param itemKey - Key of Item which the Selector represents
     */
    createSelector(selectorKey, itemKey) {
        let selector = this.getSelector(selectorKey, { notExisting: true });
        if (!this.isInstantiated) {
            internal_1.Agile.logger.warn("We recommend to use 'MY_COLLECTION.Selector' instead of 'MY_COLLECTION.createSelector' in the Collection config!");
        }
        // Check if Selector already exists
        if (selector) {
            if (!selector.isPlaceholder) {
                internal_1.Agile.logger.warn(`Selector with the name '${selectorKey}' already exists!`);
                return selector;
            }
            selector.select(itemKey, { overwrite: true });
            return selector;
        }
        // Create Selector
        selector = new internal_1.Selector(this, itemKey, {
            key: selectorKey,
        });
        this.selectors[selectorKey] = selector;
        return selector;
    }
    //=========================================================================================================
    // Has Selector
    //=========================================================================================================
    /**
     * @public
     * Check if Selector exists in Collection
     * @param selectorKey - Key/Name of Selector
     * @param config - Config
     */
    hasSelector(selectorKey, config = {}) {
        return !!this.getSelector(selectorKey, config);
    }
    //=========================================================================================================
    // Get Selector
    //=========================================================================================================
    /**
     * @public
     * Get Selector by Key/Name
     * @param selectorKey - Key/Name of Selector
     * @param config - Config
     */
    getSelector(selectorKey, config = {}) {
        config = internal_1.defineConfig(config, {
            notExisting: false,
        });
        // Get Selector
        const selector = selectorKey ? this.selectors[selectorKey] : undefined;
        // Check if Selector exists
        if (!selector || (!config.notExisting && selector.isPlaceholder))
            return undefined;
        internal_1.ComputedTracker.tracked(selector.observer);
        return selector;
    }
    //=========================================================================================================
    // Get Selector With Reference
    //=========================================================================================================
    /**
     * @public
     * Get Selector by Key/Name or a Reference to it if it doesn't exist yet
     * @param selectorKey - Name/Key of Selector
     */
    getSelectorWithReference(selectorKey) {
        let selector = this.getSelector(selectorKey, { notExisting: true });
        // Create dummy Selector to hold reference
        if (!selector) {
            selector = new internal_1.Selector(this, 'unknown', {
                key: selectorKey,
                isPlaceholder: true,
            });
            this.selectors[selectorKey] = selector;
        }
        internal_1.ComputedTracker.tracked(selector.observer);
        return selector;
    }
    //=========================================================================================================
    // Remove Selector
    //=========================================================================================================
    /**
     * @public
     * Removes Selector by Key/Name
     * @param selectorKey - Name/Key of Selector
     */
    removeSelector(selectorKey) {
        var _a;
        if (!this.selectors[selectorKey]) {
            internal_1.Agile.logger.warn(`Selector with the key/name '${selectorKey}' doesn't exist!`);
            return this;
        }
        (_a = this.selectors[selectorKey]) === null || _a === void 0 ? void 0 : _a.unselect(); // Unselects current selected Item
        delete this.selectors[selectorKey];
        return this;
    }
    //=========================================================================================================
    // Has Item
    //=========================================================================================================
    /**
     * @public
     * Check if Item exists in Collection
     * @param itemKey - Key/Name of Item
     * @param config - Config
     */
    hasItem(itemKey, config = {}) {
        return !!this.getItem(itemKey, config);
    }
    //=========================================================================================================
    // Get Item by Id
    //=========================================================================================================
    /**
     * @public
     * Get Item by Key/Name
     * @param itemKey - ItemKey of Item
     * @param config - Config
     */
    getItem(itemKey, config = {}) {
        config = internal_1.defineConfig(config, {
            notExisting: false,
        });
        // Get Item
        const item = itemKey ? this.data[itemKey] : undefined;
        // Check if Item exists
        if (!item || (!config.notExisting && !item.exists))
            return undefined;
        internal_1.ComputedTracker.tracked(item.observer);
        return item;
    }
    /**
     * @public
     * Get Item by Key/Name or a Reference to it if it doesn't exist yet
     * @param itemKey - Key/Name of Item
     */
    getItemWithReference(itemKey) {
        let item = this.getItem(itemKey, { notExisting: true });
        // Create dummy Item to hold reference
        if (!item) {
            item = new internal_1.Item(this, {
                [this.config.primaryKey]: itemKey,
                dummy: 'item',
            }, {
                isPlaceholder: true,
            });
            this.data[itemKey] = item;
        }
        internal_1.ComputedTracker.tracked(item.observer);
        return item;
    }
    //=========================================================================================================
    // Get Value by Id
    //=========================================================================================================
    /**
     * @public
     * Get Value of Item by Key/Name
     * @param itemKey - ItemKey of Item that holds the Value
     * @param config - Config
     */
    getItemValue(itemKey, config = {}) {
        const item = this.getItem(itemKey, config);
        if (!item)
            return undefined;
        return item.value;
    }
    //=========================================================================================================
    // Get All Items
    //=========================================================================================================
    /**
     * @public
     * Get all Items of Collection
     * @param config - Config
     */
    getAllItems(config = {}) {
        config = internal_1.defineConfig(config, {
            notExisting: false,
        });
        // Get Items
        const items = [];
        for (const key in this.data) {
            const item = this.data[key];
            if ((!config.notExisting && item.exists) || config.notExisting) {
                items.push(item);
            }
        }
        return items;
    }
    //=========================================================================================================
    // Get All Item Values
    //=========================================================================================================
    /**
     * @public
     * Get all Values of Items in a Collection
     * @param config - Config
     */
    getAllItemValues(config = {}) {
        const items = this.getAllItems(config);
        return items.map((item) => item.value);
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (internal_1.isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = this._key;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = internal_1.defineConfig(_config, {
            instantiate: true,
            storageKeys: [],
        });
        if (this.persistent)
            internal_1.Agile.logger.warn(`By persisting the Collection '${this._key}' twice you overwrite the old Persistent Instance!`);
        // Create persistent -> Persist Value
        this.persistent = new internal_1.CollectionPersistent(this, {
            instantiate: _config.instantiate,
            storageKeys: _config.storageKeys,
            key: key,
        });
        return this;
    }
    //=========================================================================================================
    // On Load
    //=========================================================================================================
    /**
     * @public
     * Callback Function that gets called if the persisted Value gets loaded into the Collection for the first Time
     * Note: Only useful for persisted Collections!
     * @param callback - Callback Function
     */
    onLoad(callback) {
        if (this.persistent) {
            this.persistent.onLoad = callback;
            // If Collection is already 'isPersisted' the loading was successful -> callback can be called
            if (this.isPersisted)
                callback(true);
        }
        else {
            internal_1.Agile.logger.error(`Please make sure you persist the Collection '${this._key}' before using the 'onLoad' function!`);
        }
        return this;
    }
    //=========================================================================================================
    // Get Group Count
    //=========================================================================================================
    /**
     * @public
     * Get count of registered Groups in Collection
     */
    getGroupCount() {
        let size = 0;
        for (const group in this.groups)
            size++;
        return size;
    }
    //=========================================================================================================
    // Get Selector Count
    //=========================================================================================================
    /**
     * @public
     * Get count of registered Selectors in Collection
     */
    getSelectorCount() {
        let size = 0;
        for (const selector in this.selectors)
            size++;
        return size;
    }
    //=========================================================================================================
    // Reset
    //=========================================================================================================
    /**
     * @public
     * Resets this Collection
     */
    reset() {
        var _a, _b;
        // Reset Data
        this.data = {};
        this.size = 0;
        // Reset Groups
        for (const key in this.groups)
            (_a = this.getGroup(key)) === null || _a === void 0 ? void 0 : _a.reset();
        // Reset Selectors
        for (const key in this.selectors)
            (_b = this.getSelector(key)) === null || _b === void 0 ? void 0 : _b.reset();
    }
    //=========================================================================================================
    // Put
    //=========================================================================================================
    /**
     * @public
     * Puts ItemKey/s into Group/s (GroupKey/s)
     * @param itemKeys - ItemKey/s that get added to provided Group/s
     * @param groupKeys - Group/s to which the ItemKey/s get added
     * @param config - Config
     */
    put(itemKeys, groupKeys, config = {}) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const _groupKeys = internal_1.normalizeArray(groupKeys);
        // Add ItemKeys to Groups
        _groupKeys.forEach((groupKey) => {
            var _a;
            (_a = this.getGroup(groupKey)) === null || _a === void 0 ? void 0 : _a.add(_itemKeys, config);
        });
    }
    //=========================================================================================================
    // Update Item Key
    //=========================================================================================================
    /**
     * @internal
     * Updates Key/Name of Item in all Instances (Group, Selector, ..)
     * @param oldItemKey - Old ItemKey
     * @param newItemKey - New ItemKey
     * @param config - Config
     */
    updateItemKey(oldItemKey, newItemKey, config = {}) {
        var _a;
        const item = this.getItem(oldItemKey, { notExisting: true });
        config = internal_1.defineConfig(config, {
            background: false,
        });
        if (!item || oldItemKey === newItemKey)
            return false;
        // Check if Item with newItemKey already exists
        if (this.hasItem(newItemKey)) {
            internal_1.Agile.logger.warn(`Couldn't update ItemKey from '${oldItemKey}' to '${newItemKey}' because an Item with the key/name '${newItemKey}' already exists!`);
            return false;
        }
        // Remove Item from old ItemKey and add Item to new ItemKey
        delete this.data[oldItemKey];
        this.data[newItemKey] = item;
        // Update Key/Name of Item
        item.setKey(newItemKey, {
            background: config.background,
        });
        // Update persist Key of Item (Doesn't get updated by updating key of Item because PersistKey is special formatted)
        (_a = item.persistent) === null || _a === void 0 ? void 0 : _a.setKey(internal_1.CollectionPersistent.getItemStorageKey(newItemKey, this._key));
        // Update ItemKey in Groups
        for (const groupKey in this.groups) {
            const group = this.getGroup(groupKey, { notExisting: true });
            if (!group || !group.has(oldItemKey))
                continue;
            group.replace(oldItemKey, newItemKey, { background: config.background });
        }
        // Update ItemKey in Selectors
        for (const selectorKey in this.selectors) {
            const selector = this.getSelector(selectorKey, { notExisting: true });
            if (!selector)
                continue;
            // Reselect Item in existing Selector which has selected the newItemKey
            if (selector.hasSelected(newItemKey)) {
                selector.select(newItemKey, {
                    force: true,
                    background: config.background,
                });
            }
            // Select newItemKey in existing Selector which has selected the oldItemKey
            if (selector.hasSelected(oldItemKey))
                selector.select(newItemKey, {
                    background: config === null || config === void 0 ? void 0 : config.background,
                });
        }
        return true;
    }
    //=========================================================================================================
    // Get GroupKeys That Have ItemKey
    //=========================================================================================================
    /**
     * @public
     * Gets GroupKeys that contain the passed ItemKey
     * @param itemKey - ItemKey
     */
    getGroupKeysThatHaveItemKey(itemKey) {
        const groupKeys = [];
        for (const groupKey in this.groups) {
            const group = this.getGroup(groupKey, { notExisting: true });
            if (group === null || group === void 0 ? void 0 : group.has(itemKey))
                groupKeys.push(groupKey);
        }
        return groupKeys;
    }
    //=========================================================================================================
    // Remove
    //=========================================================================================================
    /**
     * @public
     * Remove Items from Collection
     * @param itemKeys - ItemKey/s that get removed
     */
    remove(itemKeys) {
        return {
            fromGroups: (groups) => this.removeFromGroups(itemKeys, groups),
            everywhere: () => this.removeItems(itemKeys),
        };
    }
    //=========================================================================================================
    // Remove From Groups
    //=========================================================================================================
    /**
     * @public
     * Removes Item/s from Group/s
     * @param itemKeys - ItemKey/s that get removed from Group/s
     * @param groupKeys - GroupKey/s of Group/s form which the ItemKey/s will be removed
     */
    removeFromGroups(itemKeys, groupKeys) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const _groupKeys = internal_1.normalizeArray(groupKeys);
        _itemKeys.forEach((itemKey) => {
            let removedFromGroupsCount = 0;
            // Remove ItemKey from Groups
            _groupKeys.forEach((groupKey) => {
                const group = this.getGroup(groupKey, { notExisting: true });
                if (!group || !group.has(itemKey))
                    return;
                group.remove(itemKey);
                removedFromGroupsCount++;
            });
            // If Item got removed from every Groups the Item was in, remove it completely
            if (removedFromGroupsCount >=
                this.getGroupKeysThatHaveItemKey(itemKey).length)
                this.removeItems(itemKey);
        });
    }
    //=========================================================================================================
    // Remove Items
    //=========================================================================================================
    /**
     * @public
     * Removes Item completely from Collection
     * @param itemKeys - ItemKey/s of Item/s
     */
    removeItems(itemKeys) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        _itemKeys.forEach((itemKey) => {
            var _a;
            const item = this.getItem(itemKey, { notExisting: true });
            if (!item)
                return;
            // Remove Item from Groups
            for (const groupKey in this.groups) {
                const group = this.getGroup(groupKey, { notExisting: true });
                if (group === null || group === void 0 ? void 0 : group.has(itemKey))
                    group === null || group === void 0 ? void 0 : group.remove(itemKey);
            }
            // Remove Item from Storage
            (_a = item.persistent) === null || _a === void 0 ? void 0 : _a.removePersistedValue();
            // Remove Item from Collection
            delete this.data[itemKey];
            // Reselect Item in Selectors (to create new dummyItem that holds reference)
            for (const selectorKey in this.selectors) {
                const selector = this.getSelector(selectorKey, { notExisting: true });
                if (selector === null || selector === void 0 ? void 0 : selector.hasSelected(itemKey))
                    selector === null || selector === void 0 ? void 0 : selector.select(itemKey, { force: true });
            }
            this.size--;
        });
    }
    //=========================================================================================================
    // Set Data
    //=========================================================================================================
    /**
     * @internal
     * Updates existing or creates Item from provided Data
     * @param data - Data
     * @param config - Config
     */
    setData(data, config = {}) {
        const _data = internal_1.copy(data); // Transformed Data to any because of unknown Object (DataType)
        const primaryKey = this.config.primaryKey;
        config = internal_1.defineConfig(config, {
            patch: false,
            background: false,
        });
        if (!internal_1.isValidObject(_data)) {
            internal_1.Agile.logger.error(`Item Data of Collection '${this._key}' has to be an valid Object!`);
            return false;
        }
        if (!Object.prototype.hasOwnProperty.call(_data, primaryKey)) {
            internal_1.Agile.logger.error(`Collection '${this._key}' Item Data has to contain a primaryKey property called '${this.config.primaryKey}'!`);
            return false;
        }
        const itemKey = _data[primaryKey];
        let item = this.getItem(itemKey, { notExisting: true });
        const wasPlaceholder = (item === null || item === void 0 ? void 0 : item.isPlaceholder) || false;
        const createItem = !item;
        // Create or update Item
        if (!createItem && config.patch)
            item === null || item === void 0 ? void 0 : item.patch(_data, { background: config.background });
        if (!createItem && !config.patch)
            item === null || item === void 0 ? void 0 : item.set(_data, { background: config.background });
        if (createItem) {
            // Create and assign Item to Collection
            item = new internal_1.Item(this, _data);
            this.data[itemKey] = item;
            // Rebuild Groups That include ItemKey after assigning Item to Collection (otherwise it can't find Item)
            this.rebuildGroupsThatIncludeItemKey(itemKey, {
                background: config.background,
            });
        }
        // Increase size of Collection
        if (createItem || wasPlaceholder)
            this.size++;
        return true;
    }
    //=========================================================================================================
    // Rebuild Groups That Includes Item Key
    //=========================================================================================================
    /**
     * @internal
     * Rebuilds Groups that include the provided ItemKey
     * @itemKey - Item Key
     * @config - Config
     */
    rebuildGroupsThatIncludeItemKey(itemKey, config = {}) {
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: {
                enabled: true,
                exclude: [],
            },
        });
        // Rebuild Groups that include ItemKey
        for (const groupKey in this.groups) {
            const group = this.getGroup(groupKey);
            if (group === null || group === void 0 ? void 0 : group.has(itemKey)) {
                // group.rebuild(); Not necessary because a sideEffect of the Group is to rebuild it self
                group === null || group === void 0 ? void 0 : group.ingest({
                    background: config === null || config === void 0 ? void 0 : config.background,
                    force: true,
                    sideEffects: config === null || config === void 0 ? void 0 : config.sideEffects,
                    storage: false,
                });
            }
        }
    }
}
exports.Collection = Collection;
