"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Computed = void 0;
const internal_1 = require("../internal");
class Computed extends internal_1.State {
    /**
     * @public
     * Computed - Function that recomputes its value if a dependency changes
     * @param agileInstance - An instance of Agile
     * @param computeFunction - Function for computing value
     * @param config - Config
     */
    constructor(agileInstance, computeFunction, config = {}) {
        super(agileInstance, computeFunction(), {
            key: config.key,
            dependents: config.dependents,
        });
        this.deps = []; // All Dependencies of Computed (hardCoded and autoDetected)
        this.hardCodedDeps = []; // HardCoded Dependencies of Computed
        config = internal_1.defineConfig(config, {
            computedDeps: [],
        });
        this.agileInstance = () => agileInstance;
        this.computeFunction = computeFunction;
        // Format hardCodedDeps
        this.hardCodedDeps = this.formatDeps(config.computedDeps);
        this.deps = this.hardCodedDeps;
        // Recompute for setting initial value and adding missing dependencies
        this.recompute();
    }
    //=========================================================================================================
    // Recompute
    //=========================================================================================================
    /**
     * @public
     * Recomputes Value of Computed
     * @param config - Config
     */
    recompute(config = {}) {
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: {
                enabled: true,
                exclude: [],
            },
        });
        this.ingest(config);
    }
    //=========================================================================================================
    // Updates Compute Function
    //=========================================================================================================
    /**
     * @public
     * Applies new compute Function to Computed
     * @param computeFunction - New Function for computing value
     * @param deps - Hard coded dependencies of Computed Function
     * @param config - Config
     */
    updateComputeFunction(computeFunction, deps = [], config = {}) {
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: {
                enabled: true,
                exclude: [],
            },
            overwriteDeps: true,
        });
        // Update deps
        const newDeps = this.formatDeps(deps);
        if (config.overwriteDeps)
            this.hardCodedDeps = newDeps;
        else
            this.hardCodedDeps = this.hardCodedDeps.concat(newDeps);
        this.deps = this.hardCodedDeps;
        // Update computeFunction
        this.computeFunction = computeFunction;
        // Recompute for setting initial Computed Function Value and adding missing Dependencies
        this.recompute({
            background: config.background,
            sideEffects: config.sideEffects,
        });
    }
    //=========================================================================================================
    // Compute Values
    //=========================================================================================================
    /**
     * @internal
     * Computes Value and adds missing Dependencies to Computed
     */
    computeValue() {
        // Auto track Observers the computeFunction might depend on
        internal_1.ComputedTracker.track();
        const computedValue = this.computeFunction();
        const foundDeps = internal_1.ComputedTracker.getTrackedObservers();
        // Handle foundDeps and hardCodedDeps
        const newDeps = [];
        this.hardCodedDeps.concat(foundDeps).forEach((observer) => {
            newDeps.push(observer);
            // Make this Observer depending on foundDep Observer
            observer.depend(this.observer);
        });
        this.deps = newDeps;
        return computedValue;
    }
    //=========================================================================================================
    // Format Deps
    //=========================================================================================================
    /**
     * @internal
     * Gets Observer out of passed Instances
     * @param instances - Instances that hold an Observer
     */
    formatDeps(instances) {
        const finalInstances = [];
        for (const instance of instances) {
            if (instance instanceof internal_1.Observer) {
                finalInstances.push(instance);
                continue;
            }
            if (instance !== undefined &&
                instance['observer'] !== undefined &&
                instance['observer'] instanceof internal_1.Observer)
                finalInstances.push(instance['observer']);
        }
        return finalInstances;
    }
    //=========================================================================================================
    // Overwriting some functions which aren't allowed to use in Computed
    //=========================================================================================================
    patch() {
        internal_1.Agile.logger.error("You can't use patch method on ComputedState!");
        return this;
    }
    persist() {
        internal_1.Agile.logger.error("You can't use persist method on ComputedState!");
        return this;
    }
    invert() {
        internal_1.Agile.logger.error("You can't use invert method on ComputedState!");
        return this;
    }
}
exports.Computed = Computed;
