"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Runtime = void 0;
const internal_1 = require("../internal");
class Runtime {
    /**
     * @internal
     * Runtime - Performs ingested Observers
     * @param agileInstance - An instance of Agile
     */
    constructor(agileInstance) {
        // Queue system
        this.currentJob = null;
        this.jobQueue = [];
        this.notReadyJobsToRerender = new Set(); // Jobs that got performed but aren't ready to get rerendered (wait for mount)
        this.jobsToRerender = []; // Jobs that are performed and will be rendered
        this.agileInstance = () => agileInstance;
    }
    //=========================================================================================================
    // Ingest
    //=========================================================================================================
    /**
     * @internal
     * Ingests Job into Runtime that gets performed
     * @param job - Job
     * @param config - Config
     */
    ingest(job, config = {}) {
        config = internal_1.defineConfig(config, {
            perform: true,
        });
        this.jobQueue.push(job);
        // Logging
        internal_1.Agile.logger.if.tag(['runtime']).info(`Created Job '${job._key}'`, job);
        // Perform Job
        if (config.perform) {
            const performJob = this.jobQueue.shift();
            if (performJob)
                this.perform(performJob);
        }
    }
    //=========================================================================================================
    // Perform
    //=========================================================================================================
    /**
     * @internal
     * Performs Job and adds it to the rerender queue if necessary
     * @param job - Job that gets performed
     */
    perform(job) {
        this.currentJob = job;
        // Perform Job
        job.observer.perform(job);
        job.performed = true;
        // Ingest Dependents of Observer into Runtime
        job.observer.dependents.forEach((observer) => observer.ingest({ perform: false }));
        if (job.rerender)
            this.jobsToRerender.push(job);
        this.currentJob = null;
        // Logging
        internal_1.Agile.logger.if.tag(['runtime']).info(`Completed Job '${job._key}'`, job);
        // Perform Jobs as long as Jobs are left in queue, if no job left update/rerender Subscribers of jobsToRerender
        if (this.jobQueue.length > 0) {
            const performJob = this.jobQueue.shift();
            if (performJob)
                this.perform(performJob);
        }
        else {
            if (this.jobsToRerender.length > 0) {
                // https://stackoverflow.com/questions/9083594/call-settimeout-without-delay
                setTimeout(() => {
                    this.updateSubscribers();
                });
            }
        }
    }
    //=========================================================================================================
    // Update Subscribers
    //=========================================================================================================
    /**
     * @internal
     * Updates/Rerenders all Subscribed Components of the Job (Observer)
     */
    updateSubscribers() {
        if (!this.agileInstance().hasIntegration()) {
            this.jobsToRerender = [];
            this.notReadyJobsToRerender = new Set();
            return false;
        }
        if (this.jobsToRerender.length <= 0 &&
            this.notReadyJobsToRerender.size <= 0)
            return false;
        // Subscriptions that has to be updated/rerendered
        // 'Set' to combine several SubscriptionContainers that are equals into one (optimizes rerender)
        // Better would be to optimize the rerender based on the Component, because a Component can have multiple SubscriptionContainers
        const subscriptionsToUpdate = new Set();
        // Build final jobsToRerender and reset jobsToRerender Instances
        const jobsToRerender = this.jobsToRerender.concat(Array.from(this.notReadyJobsToRerender));
        this.notReadyJobsToRerender = new Set();
        this.jobsToRerender = [];
        // Check if Job Subscriptions are ready and add them to subscriptionsToUpdate
        jobsToRerender.forEach((job) => {
            job.subscriptionContainersToUpdate.forEach((subscriptionContainer) => {
                if (!subscriptionContainer.ready) {
                    this.notReadyJobsToRerender.add(job);
                    // Logging
                    internal_1.Agile.logger.warn("SubscriptionContainer/Component isn't ready to rerender!", subscriptionContainer);
                    return;
                }
                // Handle Object based Subscription
                if (subscriptionContainer.isObjectBased)
                    this.handleObjectBasedSubscription(subscriptionContainer, job);
                subscriptionsToUpdate.add(subscriptionContainer);
                job.subscriptionContainersToUpdate.delete(subscriptionContainer);
            });
        });
        subscriptionsToUpdate.forEach((subscriptionContainer) => {
            // Call 'callback function' if Callback based Subscription
            if (subscriptionContainer instanceof internal_1.CallbackSubscriptionContainer)
                subscriptionContainer.callback();
            // Call 'update method' if Component based Subscription
            if (subscriptionContainer instanceof internal_1.ComponentSubscriptionContainer)
                this.agileInstance().integrations.update(subscriptionContainer.component, this.getObjectBasedProps(subscriptionContainer));
        });
        // Logging
        internal_1.Agile.logger.if
            .tag(['runtime'])
            .info('Updated/Rerendered Subscriptions', subscriptionsToUpdate);
        return true;
    }
    //=========================================================================================================
    // Handle Object Based Subscription
    //=========================================================================================================
    /**
     * @internal
     * Finds key of Observer (Job) in subsObject and adds it to 'changedObjectKeys'
     * @param subscriptionContainer - Object based SubscriptionContainer
     * @param job - Job that holds the searched Observer
     */
    handleObjectBasedSubscription(subscriptionContainer, job) {
        let foundKey = null;
        // Check if SubscriptionContainer is Object based
        if (!subscriptionContainer.isObjectBased)
            return;
        // Find Key of Job Observer in SubscriptionContainer
        for (const key in subscriptionContainer.subsObject)
            if (subscriptionContainer.subsObject[key] === job.observer)
                foundKey = key;
        if (foundKey)
            subscriptionContainer.observerKeysToUpdate.push(foundKey);
    }
    //=========================================================================================================
    // Get Object Based Props
    //=========================================================================================================
    /**
     * @internal
     * Builds Object out of changedObjectKeys with Observer Value
     * @param subscriptionContainer - Object based SubscriptionContainer
     */
    getObjectBasedProps(subscriptionContainer) {
        var _a;
        const props = {};
        // Map trough observerKeysToUpdate and build object out of Observer value
        if (subscriptionContainer.subsObject)
            for (const updatedKey of subscriptionContainer.observerKeysToUpdate)
                props[updatedKey] = (_a = subscriptionContainer.subsObject[updatedKey]) === null || _a === void 0 ? void 0 : _a.value;
        subscriptionContainer.observerKeysToUpdate = [];
        return props;
    }
}
exports.Runtime = Runtime;
