"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Observer = void 0;
const internal_1 = require("../internal");
class Observer {
    /**
     * @internal
     * Observer - Handles subscriptions and dependencies of an Agile Class and is like an instance to the Runtime
     * Note: No stand alone class!!
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance, config = {}) {
        var _a, _b;
        this.dependents = new Set(); // Observers that depend on this Observer
        this.subs = new Set(); // SubscriptionContainers (Components) that this Observer has subscribed
        config = internal_1.defineConfig(config, {
            dependents: [],
            subs: [],
        });
        this.agileInstance = () => agileInstance;
        this._key = config.key;
        this.value = config.value;
        (_a = config.dependents) === null || _a === void 0 ? void 0 : _a.forEach((observer) => this.depend(observer));
        (_b = config.subs) === null || _b === void 0 ? void 0 : _b.forEach((subscriptionContainer) => this.subscribe(subscriptionContainer));
    }
    /**
     * @internal
     * Set Key/Name of Observer
     */
    set key(value) {
        this._key = value;
    }
    /**
     * @internal
     * Get Key/Name of Observer
     */
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Ingest
    //=========================================================================================================
    /**
     * @internal
     * Ingests Observer into Runtime
     * @param config - Configuration
     */
    ingest(config = {}) {
        config = internal_1.defineConfig(config, {
            perform: true,
            background: false,
            sideEffects: {
                enabled: true,
                exclude: [],
            },
            force: false,
        });
        // Create Job
        const job = new internal_1.RuntimeJob(this, {
            force: config.force,
            sideEffects: config.sideEffects,
            background: config.background,
            key: config.key || this._key,
        });
        this.agileInstance().runtime.ingest(job, {
            perform: config.perform,
        });
    }
    //=========================================================================================================
    // Perform
    //=========================================================================================================
    /**
     * @internal
     * Performs Job of Runtime
     * @param job - Job that gets performed
     */
    perform(job) {
        internal_1.Agile.logger.warn("Perform function isn't Set in Observer! Be aware that Observer is no stand alone class!");
    }
    //=========================================================================================================
    // Depend
    //=========================================================================================================
    /**
     * @internal
     * Adds Dependent to Observer which gets ingested into the Runtime whenever this Observer mutates
     * @param observer - Observer that will depend on this Observer
     */
    depend(observer) {
        if (!this.dependents.has(observer))
            this.dependents.add(observer);
    }
    //=========================================================================================================
    // Subscribe
    //=========================================================================================================
    /**
     * @internal
     * Adds Subscription to Observer
     * @param subscriptionContainer - SubscriptionContainer(Component) that gets subscribed by this Observer
     */
    subscribe(subscriptionContainer) {
        if (!this.subs.has(subscriptionContainer)) {
            this.subs.add(subscriptionContainer);
            // Add this to subscriptionContainer to keep track of the Observers the subscriptionContainer hold
            subscriptionContainer.subs.add(this);
        }
    }
    //=========================================================================================================
    // Unsubscribe
    //=========================================================================================================
    /**
     * @internal
     * Removes Subscription from Observer
     * @param subscriptionContainer - SubscriptionContainer(Component) that gets unsubscribed by this Observer
     */
    unsubscribe(subscriptionContainer) {
        if (this.subs.has(subscriptionContainer)) {
            this.subs.delete(subscriptionContainer);
            subscriptionContainer.subs.delete(this);
        }
    }
}
exports.Observer = Observer;
