"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubController = void 0;
const internal_1 = require("../../internal");
class SubController {
    /**
     * @internal
     * SubController - Handles subscriptions to Components
     * @param agileInstance - An instance of Agile
     */
    constructor(agileInstance) {
        this.componentSubs = new Set(); // Holds all registered Component based Subscriptions
        this.callbackSubs = new Set(); // Holds all registered Callback based Subscriptions
        this.mountedComponents = new Set(); // Holds all mounted Components (only if agileInstance.config.mount = true)
        this.agileInstance = () => agileInstance;
    }
    //=========================================================================================================
    // Subscribe with Subs Object
    //=========================================================================================================
    /**
     * @internal
     * Subscribe with Object shaped Subscriptions
     * @param integrationInstance - Callback Function or Component
     * @param subs - Initial Subscription Object
     * @param key - Key/Name of SubscriptionContainer
     */
    subscribeWithSubsObject(integrationInstance, subs = {}, key) {
        const props = {};
        // Create subsArray
        const subsArray = [];
        for (const key in subs)
            subsArray.push(subs[key]);
        // Register Subscription -> decide weather subscriptionInstance is callback or component based
        const subscriptionContainer = this.registerSubscription(integrationInstance, subsArray, key);
        // Set SubscriptionContainer to Object based
        subscriptionContainer.isObjectBased = true;
        subscriptionContainer.subsObject = subs;
        // Register subs and build props object
        for (const key in subs) {
            const observer = subs[key];
            observer.subscribe(subscriptionContainer);
            if (observer.value)
                props[key] = observer.value;
        }
        return {
            subscriptionContainer: subscriptionContainer,
            props: props,
        };
    }
    //=========================================================================================================
    // Subscribe with Subs Array
    //=========================================================================================================
    /**
     * @internal
     * Subscribe with Array shaped Subscriptions
     * @param integrationInstance - Callback Function or Component
     * @param subs - Initial Subscription Array
     * @param key - Key/Name of SubscriptionContainer
     */
    subscribeWithSubsArray(integrationInstance, subs = [], key) {
        // Register Subscription -> decide weather subscriptionInstance is callback or component based
        const subscriptionContainer = this.registerSubscription(integrationInstance, subs, key);
        // Register subs
        subs.forEach((observer) => observer.subscribe(subscriptionContainer));
        return subscriptionContainer;
    }
    //=========================================================================================================
    // Unsubscribe
    //=========================================================================================================
    /**
     * @internal
     * Unsubscribes SubscriptionContainer(Component)
     * @param subscriptionInstance - SubscriptionContainer or Component that holds an SubscriptionContainer
     */
    unsubscribe(subscriptionInstance) {
        // Helper function to remove SubscriptionContainer from Observer
        const unsub = (subscriptionContainer) => {
            subscriptionContainer.ready = false;
            // Remove SubscriptionContainers from Observer
            subscriptionContainer.subs.forEach((observer) => {
                observer.unsubscribe(subscriptionContainer);
            });
        };
        // Unsubscribe callback based Subscription
        if (subscriptionInstance instanceof internal_1.CallbackSubscriptionContainer) {
            unsub(subscriptionInstance);
            this.callbackSubs.delete(subscriptionInstance);
            // Logging
            internal_1.Agile.logger.if
                .tag(['runtime', 'subscription'])
                .info('Unregistered Callback based Subscription ', subscriptionInstance);
            return;
        }
        // Unsubscribe component based Subscription
        if (subscriptionInstance instanceof internal_1.ComponentSubscriptionContainer) {
            unsub(subscriptionInstance);
            this.componentSubs.delete(subscriptionInstance);
            // Logging
            internal_1.Agile.logger.if
                .tag(['runtime', 'subscription'])
                .info('Unregistered Component based Subscription ', subscriptionInstance);
            return;
        }
        // Unsubscribe component based Subscription with subscriptionInstance that holds a componentSubscriptionContainer
        if (subscriptionInstance.componentSubscriptionContainer) {
            unsub(subscriptionInstance.componentSubscriptionContainer);
            this.componentSubs.delete(subscriptionInstance.componentSubscriptionContainer);
            // Logging
            internal_1.Agile.logger.if
                .tag(['runtime', 'subscription'])
                .info('Unregistered Component based Subscription ', subscriptionInstance);
            return;
        }
        // Unsubscribe component based Subscription with subscriptionInstance that holds componentSubscriptionContainers
        if (subscriptionInstance.componentSubscriptionContainers &&
            Array.isArray(subscriptionInstance.componentSubscriptionContainers)) {
            subscriptionInstance.componentSubscriptionContainers.forEach((subContainer) => {
                unsub(subContainer);
                this.componentSubs.delete(subContainer);
                // Logging
                internal_1.Agile.logger.if
                    .tag(['runtime', 'subscription'])
                    .info('Unregistered Component based Subscription ', subscriptionInstance);
            });
            return;
        }
        // Logging
        internal_1.Agile.logger.if
            .tag(['runtime', 'subscription'])
            .warn(`Couldn't find anything to unregister in `, subscriptionInstance);
        return;
    }
    //=========================================================================================================
    // Register Subscription
    //=========================================================================================================
    /**
     * @internal
     * Registers SubscriptionContainer and decides weather integrationInstance is a callback or component based Subscription
     * @param integrationInstance - Callback Function or Component
     * @param subs - Initial Subscriptions
     * @param key - Key/Name of SubscriptionContainer
     */
    registerSubscription(integrationInstance, subs = [], key) {
        if (internal_1.isFunction(integrationInstance))
            return this.registerCallbackSubscription(integrationInstance, subs, key);
        return this.registerComponentSubscription(integrationInstance, subs, key);
    }
    //=========================================================================================================
    // Register Component Subscription
    //=========================================================================================================
    /**
     * @internal
     * Registers Component based Subscription and applies SubscriptionContainer to Component.
     * If an instance called 'subscriptionContainers' exists in Component it will push the new SubscriptionContainer to this Array,
     * otherwise it creates a new Instance called 'subscriptionContainer' which holds the new  SubscriptionContainer
     * @param componentInstance - Component that got subscribed by Observer/s
     * @param subs - Initial Subscriptions
     * @param key - Key/Name of SubscriptionContainer
     */
    registerComponentSubscription(componentInstance, subs = [], key) {
        const componentSubscriptionContainer = new internal_1.ComponentSubscriptionContainer(componentInstance, subs, key);
        this.componentSubs.add(componentSubscriptionContainer);
        // Set to ready if not waiting for component to mount
        if (this.agileInstance().config.waitForMount) {
            if (this.mountedComponents.has(componentInstance))
                componentSubscriptionContainer.ready = true;
        }
        else
            componentSubscriptionContainer.ready = true;
        // Add subscriptionContainer to Component, to have an instance of it there (necessary to unsubscribe SubscriptionContainer later)
        if (componentInstance.componentSubscriptionContainers &&
            Array.isArray(componentInstance.componentSubscriptionContainers))
            componentInstance.componentSubscriptionContainers.push(componentSubscriptionContainer);
        else
            componentInstance.componentSubscriptionContainer = componentSubscriptionContainer;
        // Logging
        internal_1.Agile.logger.if
            .tag(['runtime', 'subscription'])
            .info('Registered Component based Subscription ', componentSubscriptionContainer);
        return componentSubscriptionContainer;
    }
    //=========================================================================================================
    // Register Callback Subscription
    //=========================================================================================================
    /**
     * @internal
     * Registers Callback based Subscription
     * @param callbackFunction - Callback Function that causes rerender on Component which got subscribed by Observer/s
     * @param subs - Initial Subscriptions
     * @param key - Key/Name of SubscriptionContainer
     */
    registerCallbackSubscription(callbackFunction, subs = [], key) {
        const callbackSubscriptionContainer = new internal_1.CallbackSubscriptionContainer(callbackFunction, subs, key);
        this.callbackSubs.add(callbackSubscriptionContainer);
        callbackSubscriptionContainer.ready = true;
        // Logging
        internal_1.Agile.logger.if
            .tag(['runtime', 'subscription'])
            .info('Registered Callback based Subscription ', callbackSubscriptionContainer);
        return callbackSubscriptionContainer;
    }
    //=========================================================================================================
    // Mount
    //=========================================================================================================
    /**
     * @internal
     * Mounts Component based SubscriptionContainer
     * @param componentInstance - SubscriptionContainer(Component) that gets mounted
     */
    mount(componentInstance) {
        if (componentInstance.componentSubscriptionContainer)
            componentInstance.componentSubscriptionContainer.ready = true;
        this.mountedComponents.add(componentInstance);
    }
    //=========================================================================================================
    // Unmount
    //=========================================================================================================
    /**
     * @internal
     * Unmounts Component based SubscriptionContainer
     * @param componentInstance - SubscriptionContainer(Component) that gets unmounted
     */
    unmount(componentInstance) {
        if (componentInstance.componentSubscriptionContainer)
            componentInstance.componentSubscriptionContainer.ready = false;
        this.mountedComponents.delete(componentInstance);
    }
}
exports.SubController = SubController;
