"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StateObserver = void 0;
const internal_1 = require("../internal");
class StateObserver extends internal_1.Observer {
    /**
     * @internal
     * State Observer - Handles State changes, dependencies (-> Interface to Runtime)
     * @param state - State
     * @param config - Config
     */
    constructor(state, config = {}) {
        super(state.agileInstance(), Object.assign(Object.assign({}, config), { value: state._value }));
        this.state = () => state;
        this.nextStateValue = internal_1.copy(state._value);
    }
    //=========================================================================================================
    // Ingest
    //=========================================================================================================
    /**
     * @internal
     * Ingests nextStateValue or computedValue into Runtime and applies it to the State
     * @param config - Config
     */
    ingest(config = {}) {
        const state = this.state();
        let newStateValue;
        if (state instanceof internal_1.Computed)
            newStateValue = state.computeValue();
        else
            newStateValue = state.nextStateValue;
        this.ingestValue(newStateValue, config);
    }
    //=========================================================================================================
    // Ingest Value
    //=========================================================================================================
    /**
     * @internal
     * Ingests new State Value into Runtime and applies it to the State
     * @param newStateValue - New Value of the State
     * @param config - Config
     */
    ingestValue(newStateValue, config = {}) {
        const state = this.state();
        config = internal_1.defineConfig(config, {
            perform: true,
            background: false,
            sideEffects: {
                enabled: true,
                exclude: [],
            },
            force: false,
            storage: true,
            overwrite: false,
        });
        // Force overwriting State because if assigning Value to State, the State shouldn't be a placeholder anymore
        if (state.isPlaceholder) {
            config.force = true;
            config.overwrite = true;
        }
        // Assign next State Value and compute it if necessary
        this.nextStateValue = state.computeMethod
            ? internal_1.copy(state.computeMethod(newStateValue))
            : internal_1.copy(newStateValue);
        // Check if State Value and new/next Value are equals
        if (internal_1.equal(state._value, this.nextStateValue) && !config.force)
            return;
        // Create Job
        const job = new internal_1.StateRuntimeJob(this, {
            storage: config.storage,
            sideEffects: config.sideEffects,
            force: config.force,
            background: config.background,
            overwrite: config.overwrite,
            key: config.key || this._key,
        });
        this.agileInstance().runtime.ingest(job, {
            perform: config.perform,
        });
    }
    //=========================================================================================================
    // Perform
    //=========================================================================================================
    /**
     * @internal
     * Performs Job that holds this Observer
     * @param job - Job
     */
    perform(job) {
        const state = job.observer.state();
        // Assign new State Values
        state.previousStateValue = internal_1.copy(state._value);
        state._value = internal_1.copy(job.observer.nextStateValue);
        state.nextStateValue = internal_1.copy(job.observer.nextStateValue);
        // Overwrite old State Values
        if (job.config.overwrite) {
            state.initialStateValue = internal_1.copy(state._value);
            state.previousStateValue = internal_1.copy(state._value);
            state.isPlaceholder = false;
        }
        state.isSet = internal_1.notEqual(state._value, state.initialStateValue);
        this.sideEffects(job);
        // Assign Public Value to Observer after sideEffects like 'rebuildGroup',
        // because sometimes (for instance in Group) the publicValue is not the value(nextStateValue)
        // and the observer value is at some point the publicValue because the end user uses it
        job.observer.value = internal_1.copy(state.getPublicValue());
    }
    //=========================================================================================================
    // Side Effect
    //=========================================================================================================
    /**
     * @internal
     * SideEffects of Job
     * @param job - Job
     */
    sideEffects(job) {
        var _a, _b, _c;
        const state = job.observer.state();
        // Call Watchers Functions
        for (const watcherKey in state.watchers)
            if (internal_1.isFunction(state.watchers[watcherKey]))
                state.watchers[watcherKey](state.getPublicValue(), watcherKey);
        // Call SideEffect Functions
        if ((_b = (_a = job.config) === null || _a === void 0 ? void 0 : _a.sideEffects) === null || _b === void 0 ? void 0 : _b.enabled) {
            const sideEffectArray = internal_1.createArrayFromObject(state.sideEffects);
            sideEffectArray.sort(function (a, b) {
                return b.instance.weight - a.instance.weight;
            });
            for (const sideEffect of sideEffectArray) {
                if (internal_1.isFunction(sideEffect.instance.callback)) {
                    if (!((_c = job.config.sideEffects.exclude) === null || _c === void 0 ? void 0 : _c.includes(sideEffect.key)))
                        sideEffect.instance.callback(job.observer.state(), job.config);
                }
            }
        }
    }
}
exports.StateObserver = StateObserver;
