"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatePersistent = void 0;
const internal_1 = require("../internal");
class StatePersistent extends internal_1.Persistent {
    /**
     * @internal
     * State Persist Manager - Handles permanent storing of State Value
     * @param state - State that gets stored
     * @param config - Config
     */
    constructor(state, config = {}) {
        super(state.agileInstance(), {
            instantiate: false,
        });
        config = internal_1.defineConfig(config, {
            instantiate: true,
            storageKeys: [],
        });
        this.state = () => state;
        this.instantiatePersistent({
            key: config.key,
            storageKeys: config.storageKeys,
        });
        // Load/Store persisted Value for the first Time
        if (this.ready && config.instantiate)
            this.initialLoading();
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @internal
     * Updates Key/Name of Persistent
     * @param value - New Key/Name of Persistent
     */
    setKey(value) {
        return __awaiter(this, void 0, void 0, function* () {
            const oldKey = this._key;
            const wasReady = this.ready;
            // Assign Key
            if (value === this._key)
                return;
            this._key = value || internal_1.Persistent.placeHolderKey;
            const isValid = this.validatePersistent();
            // Try to Initial Load Value if persistent wasn't ready and return
            if (!wasReady) {
                if (isValid)
                    yield this.initialLoading();
                return;
            }
            // Remove value at old Key
            yield this.removePersistedValue(oldKey);
            // Assign Value to new Key
            if (isValid)
                yield this.persistValue(value);
        });
    }
    //=========================================================================================================
    // Initial Loading
    //=========================================================================================================
    /**
     * @internal
     * Loads/Saves Storage Value for the first Time
     */
    initialLoading() {
        const _super = Object.create(null, {
            initialLoading: { get: () => super.initialLoading }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.initialLoading.call(this).then(() => {
                this.state().isPersisted = true;
            });
        });
    }
    //=========================================================================================================
    // Load Persisted Value
    //=========================================================================================================
    /**
     * @internal
     * Loads State Value from the Storage
     * @return Success?
     */
    loadPersistedValue(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _key = key || this._key;
            // Load Value from default Storage
            const loadedValue = yield this.agileInstance().storages.get(_key, this.defaultStorageKey);
            if (!loadedValue)
                return false;
            // Assign loaded Value to State
            this.state().set(loadedValue, { storage: false });
            // Persist State, so that the Storage Value updates dynamically if the State updates
            yield this.persistValue(_key);
            return true;
        });
    }
    //=========================================================================================================
    // Persist Value
    //=========================================================================================================
    /**
     * @internal
     * Sets everything up so that the State gets saved in the Storage on every Value change
     * @return Success?
     */
    persistValue(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _key = key || this._key;
            // Add SideEffect to State, that updates the saved State Value depending on the current State Value
            this.state().addSideEffect(StatePersistent.storeValueSideEffectKey, (instance, config) => {
                this.rebuildStorageSideEffect(this.state(), _key, config);
            }, { weight: 0 });
            // Rebuild Storage for saving State Value in the Storage
            this.rebuildStorageSideEffect(this.state(), _key);
            this.isPersisted = true;
            return true;
        });
    }
    //=========================================================================================================
    // Remove Persisted Value
    //=========================================================================================================
    /**
     * @internal
     * Removes State Value form the Storage
     * @return Success?
     */
    removePersistedValue(key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _key = key || this._key;
            // Remove SideEffect
            this.state().removeSideEffect(StatePersistent.storeValueSideEffectKey);
            // Remove Value from Storage
            this.agileInstance().storages.remove(_key, this.storageKeys);
            this.isPersisted = false;
            return true;
        });
    }
    //=========================================================================================================
    // Format Key
    //=========================================================================================================
    /**
     * @internal
     * Formats Storage Key
     * @param key - Key that gets formatted
     */
    formatKey(key) {
        const state = this.state();
        // Get key from State
        if (!key && state._key)
            return state._key;
        if (!key)
            return;
        // Set State Key to Storage Key if State has no key
        if (!state._key)
            state._key = key;
        return key;
    }
    //=========================================================================================================
    // Rebuild Storage SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Rebuilds Storage depending on the State Value (Saves current State Value into the Storage)
     * @param state - State that holds the new Value
     * @param key - Key/Name of Persistent
     * @param config - Config
     */
    rebuildStorageSideEffect(state, key, config = {}) {
        if (config.storage !== undefined && !config.storage)
            return;
        this.agileInstance().storages.set(key, this.state().getPersistableValue(), this.storageKeys);
    }
}
exports.StatePersistent = StatePersistent;
StatePersistent.storeValueSideEffectKey = 'rebuildStateStorageValue';
