"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storages = void 0;
const internal_1 = require("../internal");
class Storages {
    /**
     * @internal
     * Storages - Manages Storages of Agile
     * @param agileInstance - An Instance of Agile
     * @param config - Config
     */
    constructor(agileInstance, config = {}) {
        this.storages = {}; // All registered Storages
        this.persistentInstances = new Set();
        this.agileInstance = () => agileInstance;
        config = internal_1.defineConfig(config, {
            localStorage: false,
        });
        if (config.localStorage)
            this.instantiateLocalStorage();
    }
    //=========================================================================================================
    // Instantiate Local Storage
    //=========================================================================================================
    /**
     * @internal
     * Instantiates Local Storage
     */
    instantiateLocalStorage() {
        // Check if Local Storage is Available
        if (!Storages.localStorageAvailable()) {
            internal_1.Agile.logger.warn('Local Storage is here not available, to use Storage functionalities like persist please provide a custom Storage!');
            return false;
        }
        // Create and register Local Storage
        const _localStorage = new internal_1.Storage({
            key: 'localStorage',
            async: false,
            methods: {
                get: localStorage.getItem.bind(localStorage),
                set: localStorage.setItem.bind(localStorage),
                remove: localStorage.removeItem.bind(localStorage),
            },
        });
        return this.register(_localStorage, { default: true });
    }
    //=========================================================================================================
    // Register
    //=========================================================================================================
    /**
     * @internal
     * Register new Storage as Agile Storage
     * @param storage - new Storage
     * @param config - Config
     */
    register(storage, config = {}) {
        const hasRegisteredAnyStorage = internal_1.notEqual(this.storages, {});
        // Check if Storage already exists
        if (Object.prototype.hasOwnProperty.call(this.storages, storage.key)) {
            internal_1.Agile.logger.error(`Storage with the key/name '${storage.key}' already exists`);
            return false;
        }
        // Set first added Storage as default Storage
        if (!hasRegisteredAnyStorage && config.default === false) {
            internal_1.Agile.logger.warn('Be aware that Agile has to assign the first added Storage as default Storage!');
        }
        if (!hasRegisteredAnyStorage)
            config.default = true;
        // Register Storage
        this.storages[storage.key] = storage;
        if (config.default)
            this.defaultStorage = storage;
        this.persistentInstances.forEach((persistent) => {
            // If Persistent isn't ready and has no default StorageKey.. reassignStorageKeys and try to load it
            if (!persistent.defaultStorageKey) {
                persistent.assignStorageKeys();
                const isValid = persistent.validatePersistent();
                if (isValid)
                    persistent.initialLoading();
                return;
            }
            // Add Value of Persistent to newly registered Storage
            if (persistent.storageKeys.includes(storage.key))
                persistent.persistValue();
        });
        return true;
    }
    //=========================================================================================================
    // Get Storage
    //=========================================================================================================
    /**
     * @internal
     * Get Storage at Key/Name
     * @param key - Key/Name of Storage
     */
    getStorage(key) {
        const storage = this.storages[key];
        // Check if Storage exists
        if (!storage) {
            internal_1.Agile.logger.error(`Storage with the key/name '${key}' doesn't exist`);
            return undefined;
        }
        // Check if Storage is ready
        if (!storage.ready) {
            internal_1.Agile.logger.error(`Storage with the key/name '${key}' isn't ready`);
            return undefined;
        }
        return storage;
    }
    //=========================================================================================================
    // Get
    //=========================================================================================================
    /**
     * @internal
     * Gets value at provided Key
     * @param key - Key of Storage property
     * @param storageKey - Key/Name of Storage from which the Item is fetched (if not provided default Storage will be used)
     */
    get(key, storageKey) {
        var _a;
        if (!this.hasStorage()) {
            internal_1.Agile.logger.error('No Storage found! Please provide at least one Storage.');
            return Promise.resolve(undefined);
        }
        // Call get Method in specific Storage
        if (storageKey) {
            const storage = this.getStorage(storageKey);
            if (storage)
                return storage.get(key);
        }
        // Call get Method in default Storage
        return ((_a = this.defaultStorage) === null || _a === void 0 ? void 0 : _a.get(key)) || Promise.resolve(undefined);
    }
    //=========================================================================================================
    // Set
    //=========================================================================================================
    /**
     * @internal
     * Saves/Updates value at provided Key
     * @param key - Key of Storage property
     * @param value - new Value that gets set at provided Key
     * @param storageKeys - Key/Name of Storages where the Value gets set (if not provided default Storage will be used)
     */
    set(key, value, storageKeys) {
        var _a, _b;
        if (!this.hasStorage()) {
            internal_1.Agile.logger.error('No Storage found! Please provide at least one Storage.');
            return;
        }
        // Call set Method in specific Storages
        if (storageKeys) {
            for (const storageKey of storageKeys)
                (_a = this.getStorage(storageKey)) === null || _a === void 0 ? void 0 : _a.set(key, value);
            return;
        }
        // Call set Method in default Storage
        (_b = this.defaultStorage) === null || _b === void 0 ? void 0 : _b.set(key, value);
    }
    //=========================================================================================================
    // Remove
    //=========================================================================================================
    /**
     * @internal
     * Removes value at provided Key
     * @param key - Key of Storage property
     * @param storageKeys - Key/Name of Storages where the Value gets removed (if not provided default Storage will be used)
     */
    remove(key, storageKeys) {
        var _a, _b;
        if (!this.hasStorage()) {
            internal_1.Agile.logger.error('No Storage found! Please provide at least one Storage.');
            return;
        }
        // Call remove Method in specific Storages
        if (storageKeys) {
            for (const storageKey of storageKeys)
                (_a = this.getStorage(storageKey)) === null || _a === void 0 ? void 0 : _a.remove(key);
            return;
        }
        // Call remove Method in default Storage
        (_b = this.defaultStorage) === null || _b === void 0 ? void 0 : _b.remove(key);
    }
    //=========================================================================================================
    // Has Storage
    //=========================================================================================================
    /**
     * @internal
     * Check if at least one Storage got registered
     */
    hasStorage() {
        return internal_1.notEqual(this.storages, {});
    }
    //=========================================================================================================
    // Local Storage Available
    //=========================================================================================================
    /**
     * @internal
     * Checks if localStorage is available in this Environment
     */
    static localStorageAvailable() {
        try {
            localStorage.setItem('_myDummyKey_', 'myDummyValue');
            localStorage.removeItem('_myDummyKey_');
            return true;
        }
        catch (e) {
            return false;
        }
    }
}
exports.Storages = Storages;
